An agent to execute Talend Open Studio for Data Integration Jobs.

# Talend Open Studio Active Workflow Agent

This Agents complies to the [external REST-API](https://github.com/automaticmode/active_workflow/blob/master/docs/remote_agent_api.md) of [ActiveWorkflow](https://github.com/automaticmode/active_workflow). It allows to configure built Talend Open Studio Jobs that can be run using a REST call.

## API Caller Configurations

### Register

Registering of this agent works by setting a Job and its Version. The Job has to be specified under `job_name` and the according version under `job_version`. Under `context`, a dictionary of key value pairs is expected. Setting `context` to `{}` will lead to no context parameters to be given to the Job, letting it run with its default context. In order to catch an output of the job itself (apart from generated statistics, see check for more details), you may supply the `talend_output_context_param` configuration parameter. This parameter specifies the key to a context variable which value will be set to a point to a JSON file. This JSON file can be used by a TOS Job to write anything into it, which will be read and returned to ActiveWorkflow after the Job run is completed.

The exact names and version known to the agent can be found here (keep the header intact to allow the agent to replace it):

#### Joblist


#### Example

```
{
  "job_name": "DummyJob",
  "job_version": "1.0",
  "talend_output_context_param": "aw_message_filepath",
  "context": {
    "test_val": 8
  }
}
```

### Receive

Receiving messages allows to override context variables in its message body. Other than that, receiving should be done completely by ActiveWorkflow. Started Job runs will be returned as `jobs` in the memory section of a result, allowing ActiveWorkflow to manage the state of the agent. A successful receive will start a Job run in the background, return a 202 Accepted HTTP message including an updated memory (including the job_run_id) and wait for a check message to return the result if ready.

#### Example

Full request:
```
{
    "method": "receive",
    "params": {
        "message": {
            "payload": {
                "context": {
                    "test_val": 12
                }
            }
        },
        "options": {
            "job_name": "DummyJob",
            "job_version": "1.0",
            "context": {
                "test_val": 8
            }
        }, 
        "memory": {
            "jobs": []
        },
        "credentials": {}
    }
}
```

Full response:
```
{
    "result": {
        "errors": [],
        "logs": ["Job DummyJob has been started with id 21."],
        "memory": {
            "jobs": [21]
        },
        "messages": [],
    }
}
```

### Check

Checking on Job runs. All jobs listed in `$.params.memory.jobs` are checked, where the `$.params.options` are cross checked aswell (`job_name` and `job_version` have to match the id in `$.params.memory.jobs`). Each running Job will be responded with a log message. A completed Job will add its stdout and stderr output to `$.results.logs` and `$.results.errors` respectively. In the message part, stats about a completed Job will be returned alongside any response from the Job itself via `talend_output_context_param` under `$.results.messages[0].response`.

#### Example

Full request:
```
{
    "method": "check",
    "params": {
        "message": null
        "options": {
            "job_name": "DummyJob",
            "job_version": "1.0",
            "context": {
                "test_val": 8
            }
        },
        "memory": {
            "jobs": [21]
        },
        "credentials": [],
    }
}
```

Full response:
```
{
    "result": {
        "errors": [],
        "logs": [
            "25|2pdwaA|QvVW2g6IdzHS|key.compareTo(25) != 0 failed\n"
        ],
        "memory": {
            "jobs": []
        },
        "messages": [
            {
                "job_run_id": 21,
                "job_id": 4,
                "job_name": "DummyJob",
                "job_version": "1.0",
                "completed": true,
                "returncode": 0,
                "runtime": 1.036232,
                "response": {
                    "data": [
                        {
                            "entity": 1,
                            "attribute": "wARNk5",
                            "value": "7riXpzHPFECz"
                        }
                    ]
                }
            }
        ]
    }
}
```

### Additional API Calls

You can call the `/joblist` endpoint to find a list of all available jobs. This endpoint also refreshes the list of jobs available to the agent. As this information is cached by ActiveWorkflow, it may be newer compared to the list seen in ActiveWorkflow.

## Service Configuration

Configuration can be done using environment variables. It does not matter whether done locally or given to docker.

* `TALEND_BASE_FOLDER`: the folder that holds the Talend Open Studio jobs, e.g. the built .zip files. Keep in mind that this folder is in the docker container if you run the agent dockerized. Default: `"/etc/talend"`
* `TALEND_OUTPUT_CONTEXT_PARAM`: a context parameter that indicates a filepath where an output message is written by a TOS job. Allows to capture an output from TOS jobs. Default: `""`
* `DB_URI`: Path to a database (tested with postgres-12) in the [sqlalchemy notation](https://docs.sqlalchemy.org/en/13/core/engines.html#database-urls): `postgresql://<user>:<pass>@<host>:<port>/<db>`. Default: `"postgresql://tos:agent@localhost:5432/tosagent"`

Further configurations that may be less relevant. These settings most likely do not need to be modified.

* `APPLICATION_NAME`: the name of the agent that will be published in its API. Default: `"Talend Open Studio Executor Agent"`
* `APPLICATION_VERSION`: the version of the agent that will be published in its API. Default is the current version of the application.
* `TALEND_JOB_CACHE`: the internal cache where jobs from `TALEND_BASE_FOLDER` will be put. Default: `"/tmp/jobs"`

### Usage

`pipenv shell`  
`uvicorn tos_execute.main:app --host 0.0.0.0`

or

`pip install -e requirements.txt`  
`uvicorn tos_execute.main:app --host 0.0.0.0`

or

`docker build -t myimage .`  
`docker run -d -p 8000:80 myimage`

Find your app at `localhost:8000`, you can verify the app is running by checking the docs under `localhost:8000/docs` or `localhost:8000/redoc`.

### ActiveWorkflow

After starting Talend Open Studio Active Workflow Agent, you can supply the `tosexecutor` endpoint as an environment variable like `REMOTE_AGENT_URL=http://localhost:8000/tosexecutor`