import enum

from sqlalchemy import (
    Boolean,
    Integer,
    Column,
    Enum,
    DateTime,
    String,
    JSON,
    ForeignKey,
    UniqueConstraint,
)
from sqlalchemy.orm import relationship
from tos_execute.database import Base


class JobState(enum.Enum):
    started = 1
    completed = 2
    retrieved = 3


class Job(Base):
    __tablename__ = "job"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String, index=True)
    version = Column(String, index=True)
    path = Column(String)
    available = Column(Boolean)

    UniqueConstraint("name", "version", name="job_unique_constraint")

    runs = relationship("JobRun", back_populates="job")


class JobRun(Base):
    __tablename__ = "job_run"

    id = Column(Integer, primary_key=True, index=True)
    job_id = Column(Integer, ForeignKey("job.id"), index=True)
    state = Column(Enum(JobState), index=True)
    started_at = Column(DateTime)
    completed_at = Column(DateTime)
    retrieved_at = Column(DateTime)
    response = Column(JSON)

    job = relationship("Job", back_populates="runs")
