import datetime

from typing import List, Optional

from sqlalchemy import and_, update
from sqlalchemy.orm import Session

from tos_execute import database
from tos_execute import models_internal as intmodel


def get_db():
    db = database.SessionLocal()
    try:
        yield db
    finally:
        db.close()


def get_jobs_by_availability(avail: bool, db: Session) -> List[intmodel.Job]:
    return db.query(intmodel.Job).filter(intmodel.Job.available == avail).all()


def get_job(name: str, version: str, db: Session) -> Optional[intmodel.Job]:
    return (
        db.query(intmodel.Job)
        .filter(and_(intmodel.Job.name == name, intmodel.Job.version == version))
        .first()
    )


def set_all_jobs_inactive(db: Session) -> None:
    update(intmodel.Job).where(intmodel.Job.available == True).values(available=False)


def store_job(job: intmodel.Job, db: Session) -> intmodel.Job:
    db.add(job)
    db.commit()
    db.refresh(job)
    return job


def start_job_run(job: intmodel.Job, db: Session) -> intmodel.JobRun:
    job_run = intmodel.JobRun(
        job=job, state=intmodel.JobState.started, started_at=datetime.datetime.now()
    )
    db.add(job_run)
    db.commit()
    db.refresh(job_run)
    return job_run


def store_job_run(job_run: intmodel.JobRun, db: Session) -> intmodel.JobRun:
    db.add(job_run)
    db.commit()
    db.refresh(job_run)
    return job_run


def get_job_run_by_id(job_id: int, db: Session) -> Optional[intmodel.JobRun]:
    return db.query(intmodel.JobRun).filter(intmodel.JobRun.id == job_id).first()
